/*
 * Copyright (C) 2004-2013 L2J DataPack
 * 
 * This file is part of L2J DataPack.
 * 
 * L2J DataPack is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * L2J DataPack is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
package ai.npc.Teleports.TeleportToRaceTrack;

import ai.npc.AbstractNpcAI;

import com.l2jserver.gameserver.model.Location;
import com.l2jserver.gameserver.model.actor.L2Npc;
import com.l2jserver.gameserver.model.actor.instance.L2PcInstance;
import com.l2jserver.gameserver.model.quest.QuestState;
import com.l2jserver.gameserver.model.quest.State;
import com.l2jserver.gameserver.network.serverpackets.NpcSay;
import com.l2jserver.gameserver.util.Util;

/**
 * Race Track teleport AI.<br>
 * Original python script by DraX & updated by DrLecter.
 * @author Plim
 */
public class TeleportToRaceTrack extends AbstractNpcAI
{
	// NPC
	private static final int RACE_MANAGER = 30995;
	// Misc
	//@formatter:off
	private static final int[] TELEPORTERS =
	{
	/*id*/    -1,
	/* 1*/ 30320,	// RICHLIN
	/* 2*/ 30256,	// BELLA
	/* 3*/ 30059,	// TRISHA
	/* 4*/ 30080,	// CLARISSA
	/* 5*/ 30899,	// FLAUEN
	/* 6*/ 30177,	// VALENTIA
	/* 7*/ 30848,	// ELISA
	/* 8*/ 30233,	// ESMERALDA
	/* 9*/ 31320,	// ILYANA
	/*10*/ 31275,	// TATIANA
	/*11*/ 31964,	// BILIA
	/*12*/ 31210,	// RACE TRACK GK
	};
	// Locations
	private static final Location[] RETURN_LOCS =
	{
	/*id*/ null,
	/* 1*/ new Location(-80826, 149775, -3043),
	/* 2*/ new Location(-12672, 122776, -3116),
	/* 3*/ new Location(15670, 142983, -2705),
	/* 4*/ new Location(83400, 147943, -3404),
	/* 5*/ new Location(111409, 219364, -3545),
	/* 6*/ new Location(82956, 53162, -1495),
	/* 7*/ new Location(146331, 25762, -2018),
	/* 8*/ new Location(116819, 76994, -2714),
	/* 9*/ new Location(43835, -47749, -792),
	/*10*/ new Location(147930, -55281, -2728),
	/*11*/ new Location(87386, -143246, -1293),
	/*12*/ new Location(12882, 181053, -3560)
	};
	//@formatter:on
	
	@Override
	public String onTalk(L2Npc npc, L2PcInstance player)
	{
		String htmltext = "";
		QuestState st = player.getQuestState(getName());
		
		if (st == null)
		{
			return null;
		}
		
		int teleport_id;
		int return_id;
		if ((teleport_id = Util.indexOf(TELEPORTERS, npc.getNpcId())) > 0)
		{
			st.getPlayer().teleToLocation(12661, 181687, -3560);
			st.setState(State.STARTED);
			st.set("id", String.valueOf(teleport_id));
		}
		else if (npc.getNpcId() == RACE_MANAGER)
		{
			if (st.getState() == State.STARTED && (return_id = st.getInt("id")) > 0 && return_id < RETURN_LOCS.length)
			{
				st.getPlayer().teleToLocation(RETURN_LOCS[return_id], false);
			}
			else
			{
				player.sendPacket(new NpcSay(npc.getObjectId(), 0, npc.getNpcId(), "H܂ňႤ@ŗ́HꂶA߂̃fCȊ֑ĂˁB"));
			//	player.sendPacket(new NpcSay(npc.getObjectId(), 0, npc.getNpcId(), "You've arrived here from a different way. I'll send you to Dion Castle Town which is the nearest town."));
				st.getPlayer().teleToLocation(15670, 142983, -2700);
			}
			st.unset("id");
			st.exitQuest(true);
		}
		return htmltext;
	}
	
	private TeleportToRaceTrack(String name, String descr)
	{
		super(name, descr);
		addStartNpc(TELEPORTERS);
		addTalkId(TELEPORTERS);
		
		addStartNpc(RACE_MANAGER);
		addTalkId(RACE_MANAGER);
	}
	
	public static void main(String[] args)
	{
		new TeleportToRaceTrack(TeleportToRaceTrack.class.getSimpleName(), "ai/npc/Teleports/");
	}
}
